const { ReactComponent, useEffect } = wp.element;
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import axios from 'axios';
import Masonry from 'react-masonry-css';
import CheckboxType from './types/CheckboxType';
import SelectType from './types/SelectType';
import RadioType from './types/RadioType';
import Searchbar from './types/Searchbar';
import SinglePost from './SinglePost';
import Pagination from './Pagination';
import qs from 'query-string';
import StickyBox from "react-sticky-box";

class Filter extends Component {
    constructor( props ) {
        super( props );
        
        this.state = {
            filterData: this.props,
            results: {},
            filters: [],
            query: {},
            currentPage: 1,
            totalPages: 0,
            message: ''
        }

        this.cancel = '';
    };

    static PropTypes = {
        post: PropTypes.object.isRequired
    }

    async componentDidMount() {
        this.fetchSearchResults();
        this.fetchFilterResults();
    }

    getPageCount = ( total, denominator ) => {
        const divisible = 0 === total % denominator;
        const valueToBeAdded = divisible ? 0 : 1;
        return Math.floor(total/denominator) + valueToBeAdded;
    };

    fetchSearchResults = () => {
        const { query, filterData, currentPage } = this.state;
        const { field_relationships, order_by, posts_per_page, post_types } = filterData.data;
        const search_post_type = (post_types != 'posts') ? post_types : 'posts';
        const posts_per_page_q = ( posts_per_page ) ? parseInt(posts_per_page) : 10;
        query['per_page'] = posts_per_page_q;
        query['page'] = parseInt(currentPage);
        query['tax_relation'] = ( field_relationships ) ? field_relationships : 'AND';
        query['orderby'] = ( order_by ) ? order_by : 'date';
        const searchString = qs.stringify(query);

        let searchUrl = `${main_site_url.url}/wp-json/wp/v2/${search_post_type}?${searchString}`;
        if( permalink_structure.val == '' ){
            searchUrl = `${main_site_url.url}/?rest_route=/wp/v2/${search_post_type}&${searchString}`;
        }
        if( this.cancel ){
            this.cancel.cancel();
        }
        this.cancel = axios.CancelToken.source();

        axios.get(searchUrl, {
            cancelToken: this.cancel.token
        } ).then( res => {
            const total = res.headers['x-wp-total'];
            const totalPagesCount = this.getPageCount(total, posts_per_page_q);
            this.setState({
                results: res.data,
                totalPages: totalPagesCount
            });
        } )
        .catch( error => {
            if( axios.isCancel(error) || error ){
                this.setState({
                    message: 'Failed data.'
                })
            }
        } );
    }

    fetchFilterResults = ( type ) => {
        const { filterData } = this.state;
        const { filters } = filterData.data;
        const requests = [];
        const requestData = [];

        {filters.map(result => {
            const {filter_options, filter_selection, filter_name} = result;
            if( filter_options != 'search' ){
                let searchUrl = `${main_site_url.url}/wp-json/wp/v2/${filter_options}?per_page=100`;
                if( permalink_structure.val == '' ){
                    searchUrl = `${main_site_url.url}/?rest_route=/wp/v2/${filter_options}&per_page=100`;
                }
                const singleR = axios.get(searchUrl);
                Array.prototype.push.apply(requests, [singleR]);
                Array.prototype.push.apply(requestData, [{'option': filter_options, 'name': filter_name, 'type': filter_selection}]);
            } else if(filter_options == 'search') {
                const filt = this.state.filters;
                Array.prototype.push.apply(filt, [{'option': filter_options, 'name': filter_name, 'type': filter_options}]);
                this.setState({
                    filters: filt,
                });
            }
        } ) }

        axios.all(requests).then(axios.spread((...responses) => {
            if ( Object.keys(responses).length && responses.length ) {
                responses.map( (result, index) => {
                    const {data} = result;
                    const {option, name, type} = requestData[index];
                    let newArr = [{
                        'type': type,
                        'option': option,
                        'name': name,
                        'data': data
                    }];

                    const { filters } = this.state;
                    Array.prototype.push.apply(filters, newArr);
                    this.setState({
                        filters: filters,
                    });
                });
            }
        })).catch(errors => {
            this.setState({
                message: 'Failed data.'
            })
        });
    }

    handleSelect = ( event ) => {
        const { query } = this.state;
        const val = event.target.value;
        const name = event.target.name;
        query[name] = val;

        this.setState( {query: query, currentPage: 1}, () => {
            this.fetchSearchResults();
        } );
    }

    handleRadio = ( event ) => {
        const { query } = this.state;
        const val = event.target.value;
        const name = event.target.name;
        if( val != '' ){
            query[name] = val;
        } else {
            query.hasOwnProperty(name) // true
                delete query[name]
        }

        this.setState( {query: query, currentPage: 1}, () => {
            this.fetchSearchResults();
        } );
    }

    handleCheck = ( event ) => {
        const { query } = this.state;
        const name = event.target.name;
        const val = event.target.value;
        let oldV = ( query[name] !== undefined ) ? query[name] : '';
        if(oldV != ''){
            oldV = oldV.split(",");
        } else {
            oldV = [];
        }
        
        if( event.target.checked ){
            Array.prototype.push.apply(oldV, [val]);
        } else {
            var index = oldV.indexOf(val);
            if (index !== -1) {
                oldV.splice(index, 1);
            }
        }
        oldV = oldV.join();
        if( oldV != '' ){
            query[name] = oldV;
        } else {
            query.hasOwnProperty(name) // true
                delete query[name]
        }

        this.setState( {query: query, currentPage: 1}, () => {
            this.fetchSearchResults();
        } );
    }

    handleRefresh = () => {
        this.setState( {query: {}, currentPage: 1}, () => {
            this.fetchSearchResults();
        } );
    }

    handlePage = ( event ) => {
        const page = event.target.getAttribute('data-page');
        if(page !== null){
            this.setState( {currentPage: page}, () => {
                this.fetchSearchResults();
            } );
        }
    }

    handleSearch = ( event ) => {
        const { query } = this.state;
        const val = event.target.value;
        const name = event.target.name;
        query[name] = val;

        this.setState( {query: query, currentPage: 1}, () => {
            this.fetchSearchResults();
        } );
    }

    renderFilterResults = () => {
        const { filters, filterData, query } = this.state;
        const { reset_button } = filterData.data;
        const { filterid } = filterData;
        const { __ } = wp.i18n;
        if ( Object.keys(filters).length && filters.length ) {
            return (
                <div className="fff-filter-sidebar">
                    {reset_button == 'yes' &&
                        <button className="fff-refresh-button" onClick={this.handleRefresh}>
                            <span>{__( 'Refresh', 'fffilter' )}</span>
                            <img src={`${main_plugin_url.url}/assets/img/refresh.svg`} />
                        </button>
                    }

                    { filters.map( result => {
                        const { type, option, data, name } = result;
                        let queryOption = '';
                        if( query[option] ){
                            queryOption = query[option];
                        }
                        if( type == 'checkbox' ){
                            return (
                                <CheckboxType handleCheck={this.handleCheck} query={queryOption} mainfilter={filterid} name={name} option={option} filter={data} />
                            );
                        } else if ( type == 'select' ) {
                            return (
                                <SelectType handleSelect={this.handleSelect} query={queryOption} mainfilter={filterid} name={name} option={option} filter={data} />
                            );
                        } else if ( type == 'radio' ) {
                            return (
                                <RadioType handleRadio={this.handleRadio} query={queryOption} mainfilter={filterid} name={name} option={option} filter={data} />
                            );
                        } else if ( type == 'search' ) {
                            return (
                                <Searchbar handleSearch={this.handleSearch} query={queryOption} option={option} name={name} />
                            );
                        }
                    } ) }
                </div>
            )
        }
    }

    renderSearchResults = () => {
        const { results, filterData } = this.state;
        const { data } = filterData;
        if ( Object.keys(results).length && results.length ) {
            if( data.posts_view == 'masonry' ){
                return (
                    <Masonry
                        breakpointCols={2}
                        className="fff-masonry-grid"
                        columnClassName="fff-sinle-post-grid">
                        { results.map( result => {
                            return (
                                <SinglePost key={result.id} filterdata={filterData.data} postdata={result} />
                            )
                        } ) }

                    </Masonry>
                )
            } else {
                return (
                    <div className="fff-classic-grid">
                        { results.map( result => {
                            return (
                                <SinglePost key={result.id} filterdata={filterData.data} postdata={result} />
                            )
                        } ) }
                    </div>
                )
            }
        }
    }

    render() {
        const { filterData, totalPages, currentPage } = this.state;
        const { data } = filterData;
        const sticky = data.sticky_sidebar;
        const pagination = data.pagination;
        let topOffset = 15;
        if( document.querySelectorAll('#wpadminbar').length != 0 ){
            topOffset = 40;
        }
        if(data.filter_position == 'right'){
            return(
                <div className={`fff-filter-wrap ${data.filter_position}`}>
                    <div className="fff-search-res">
                        {this.renderSearchResults()}
                        { pagination == 'yes' && <Pagination total={totalPages} current={currentPage} cl={this.handlePage} /> }
                    </div>
                    <div className="fff-filter-sidebar-cont">
                        {sticky == 'yes' && <StickyBox offsetTop={topOffset} offsetBottom={15}>
                            {this.renderFilterResults()}
                        </StickyBox> }
                        {sticky == 'no' && this.renderFilterResults()}
                    </div>
                </div>
            )
        } else {
            return(
                <div className={`fff-filter-wrap ${data.filter_position}`}>
                    <div className="fff-filter-sidebar-cont">
                        {sticky == 'yes' && <StickyBox offsetTop={topOffset} offsetBottom={15}>
                            {this.renderFilterResults()}
                        </StickyBox> }
                        {sticky == 'no' && this.renderFilterResults()}
                    </div>
                    <div className="fff-search-res">
                        {this.renderSearchResults()}
                        { pagination == 'yes' && <Pagination total={totalPages} current={currentPage} cl={this.handlePage} /> }
                    </div>
                </div>
            )
        }
    }
}

export default Filter;