const { Component } = wp.element;
import PropTypes from 'prop-types';
import axios from 'axios';
import { Parallax, Background } from 'react-parallax';

class SinglePost extends Component {
    state = {
        postData: {},
        filterData: {},
        categories: {},
        author: {},
        featureImg: '',
        isLoaded: false
    }

    static PropTypes = {
        post: PropTypes.object.isRequired
    }

    async componentDidMount() {
        const { postdata, filterdata } = this.props;
        const { id, author, categories, featured_media } = postdata;
        if(id){
            const allRequests = [];

            let permSt = '/wp-json/wp/v2/';
            if( permalink_structure.val == '' ){
                permSt = '/?rest_route=/wp/v2/';
            }

            if( author !== undefined ){
                const postAuthorLink = `${main_site_url.url}${permSt}users/${author}`;
                const postAuthorReq = axios.get(postAuthorLink);
                Array.prototype.push.apply(allRequests, [postAuthorReq]);
            }

            if( featured_media != 0 ){
                const postMediaLink = `${main_site_url.url}${permSt}media/${featured_media}`;
                const postMediaReq = axios.get(postMediaLink);
                Array.prototype.push.apply(allRequests, [postMediaReq]);
                this.setState({ featureImg: featured_media });
            }
            
            if ( categories !== undefined && Object.keys(categories).length && categories.length ) {
                categories.map( (categoriesRes, index) => {
                    const postCatLink = `${main_site_url.url}${permSt}categories/${categoriesRes}`;
                    const singleR = axios.get(postCatLink);
                    Array.prototype.push.apply(allRequests, [singleR]);
                });
            }

            const results = await Promise.all(allRequests.map(p => p.catch(e => e)));
            if ( Object.keys(results).length && results.length ) {
                let postAuthorData = {link: '', name: ''};
                let postMediaData = '';
                let postCatsData = [];
                results.map( (result, index) => {
                    if( index == 0 ){
                        if( !(result instanceof Error) ){
                            postAuthorData = result.data;
                        }
                    }
                    
                    const { featureImg } = this.state;
                    if( featureImg != '' && index == 1 ){
                        if( !(result instanceof Error) ){
                            if( result.data.media_details !== undefined ){
                                if( result.data.media_details.sizes.medium_large !== undefined ){
                                    postMediaData = result.data.media_details.sizes.medium_large.source_url;
                                } else {
                                    postMediaData = result.data.media_details.sizes.full.source_url;
                                }
                            }
                        }
                    }

                    if( (featureImg == '' && index > 0) || (featureImg != '' && index > 1) ){
                        if( !(result instanceof Error) ){
                            Array.prototype.push.apply(postCatsData, [result.data]);
                        }
                    }
                });

                this.setState({
                    postData: postdata,
                    filterData: filterdata,
                    author: postAuthorData,
                    categories: postCatsData,
                    featureImg: postMediaData,
                    isLoaded: true
                });
            }
        }
    }

    renderCategories = () => {
        const { categories } = this.state;
        if ( Object.keys(categories).length && categories.length ) {
            return (
                <div className="fff-post-category-list">
                    { categories.map( category => {
                        return (
                            <a href={category.link}>{category.name}</a>
                        )
                    } ) }
                </div>
            )
        }
    }

    renderMeta = () => {
        const { author } = this.state;
        const { __ } = wp.i18n;
        return (
            <div className="fff-sinle-post-meta">
                <p>{__( 'Posted by', 'fffilter' )} <a href={author.link}>{author.name}</a></p>
            </div>
        )
    }

    renderMedia = () => {
        const { postData, featureImg, filterData } = this.state;
        if(featureImg != ''){
            if(filterData.posts_view == 'masonry'){
                return (
                    <div className="fff-featured">
                        <a href={postData.link}>
                            <img src={featureImg} />
                        </a>
                    </div>
                )
            } else {
                return (
                    <Parallax bgImage={featureImg} strength={-200}>
                        <div style={{ height: 400 }}></div>
                    </Parallax>
                )
            }
        }
    }

    render() {
        const { postData, filterData, isLoaded } = this.state;
        const title = (postData.title !== undefined) ? postData.title.rendered : '';
        const exc = (postData.excerpt !== undefined) ? postData.excerpt.rendered : '';
        if(isLoaded){
            return(
                <div className="fff-sinle-post">
                    {filterData.single_feature_image == 'yes' && this.renderMedia()}
                    <div className="fff-sinle-post-cont">
                        {filterData.single_post_meta == 'yes' && this.renderCategories()}
                        {filterData.single_post_title == 'yes' && 
                        <a href={postData.link}>
                            <h4 dangerouslySetInnerHTML={{__html: title}}></h4>
                        </a>
                        }
                        {filterData.single_post_excerpt == 'yes' &&
                        <div dangerouslySetInnerHTML={{__html: exc}}></div>
                        }
                        {filterData.single_post_meta == 'yes' && this.renderMeta()}
                    </div>
                </div>
            )
        }

        return null;
    }
}

export default SinglePost;