<?php
/*
   Plugin Name: Fast & Fancy Filter - 3F
   Description: Post search filter using WordPress REST API.
   Author: WebArea
   Version: 1.2.2
   Text Domain: fffilter
   Domain Path: /languages
   License: GPLv2
*/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! class_exists( 'FastFancyFilter' ) ) :
    /**
	 * Main FastFancyFilter Class.
	 *
	 * @class   FastFancyFilter
	 * @version 1.0.0
	 */
	final class FastFancyFilter {
        /**
		 * Instance of this class.
		 *
		 * @var object
		 */
        protected static $_instance = null;
        
        /**
		 * Return an instance of this class.
		 *
		 * @return object A single instance of this class.
		 */
		public static function instance() {
			// If the single instance hasn't been set, set it now.
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}

			return self::$_instance;
        }

        /**
		 * FastFancyFilter Constructor.
		 */
		public function __construct() {
            $this->define_constants();
            $this->includes();
        }

        /**
		 * Define Constants.
		 */
		private function define_constants() {
			$this->define( 'FFF_PLUGIN_FILE', __FILE__ );
            $this->define( 'FFF_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
            $this->define( 'FFF_PLUGIN_URL', plugins_url( '' , __FILE__ ) );
            $this->define( 'FFF_DS', DIRECTORY_SEPARATOR );
			$this->define( 'FFF_ABSPATH', dirname( __FILE__ ) . FFF_DS );
			$this->define( 'FFF_VERSION', '1.0.0' );

			load_plugin_textdomain( 'fffilter', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}

        /**
		 * Define constant if not already set.
		 *
		 * @param string      $name
		 * @param string|bool $value
		 */
		private function define( $name, $value ) {
			if ( ! defined( $name ) ) {
				define( $name, $value );
			}
		}

        /**
		 * Includes.
		 */
		private function includes() {
			if( is_admin() ){
				include_once FFF_ABSPATH . 'includes/admin/class-admin.php';
			}

			include_once FFF_ABSPATH . 'includes/front/class-front.php';
        }
    }

    /**
     * Returns one instance
     *
     * @since 1.0.0
     * @return object
     */
    function FastFancyFilterFunction() {
        return FastFancyFilter::instance();
    }

    FastFancyFilterFunction();
endif;