<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class FFFilter_Admin {
    public $admin_page_url;

    /**
	 * Class Constructor.
	 */
	public function __construct() {
        // Link to settings page
        add_filter( 'plugin_action_links_'.FFF_PLUGIN_BASENAME, array( $this, 'fff_page_settings_link' ) );

        // Enqueue scripts
        add_action( 'admin_enqueue_scripts', array( $this, 'fff_admin_enqueue_scripts' ) );

        // Add settings page
        add_action( 'admin_menu', array( $this, 'fff_add_menu_page' ) );

        // Save options
        add_action( 'wp_ajax_fff_save_settins', array($this, 'saveFields') );
        add_action( 'wp_ajax_nopriv_fff_save_settins', array($this, 'saveFields') );

        // Remove filter
        add_action( 'wp_ajax_fff_remove_filter', array($this, 'removeFilter') );
        add_action( 'wp_ajax_nopriv_fff_remove_filter', array($this, 'removeFilter') );

        // Get taxonomies
        add_action( 'wp_ajax_fff_get_taxonomies', array($this, 'getTaxonomies') );
        add_action( 'wp_ajax_nopriv_fff_get_taxonomies', array($this, 'getTaxonomies') );

        $this->admin_page_url = admin_url('admin.php?page=fff-settings-page');
    }

    public function fff_page_settings_link( $links ){
        $links[] = '<a href="' . admin_url( 'admin.php?page=fff-settings-page' ) . '">' . __( 'Settings', 'fffilter' ) . '</a>';
	    return $links;
    }

    public function fff_admin_enqueue_scripts( $hook ){
        wp_register_style( 'fffilter_wp_icon_css', FFF_PLUGIN_URL . '/assets/css/icon-style.css' );
        wp_register_style( 'fffilter_wp_admin_css', FFF_PLUGIN_URL . '/assets/css/admin-style.css' );
        wp_enqueue_style( 'fffilter_wp_icon_css' );
        if($hook === 'toplevel_page_fff-settings-page' || $hook === '3f-filters_page_fff-settings-general-page'){
            wp_enqueue_code_editor( array( 'type' => 'text/html' ) );
            wp_enqueue_script( 'wp-color-picker' );
	        wp_enqueue_style( 'wp-color-picker' );
            wp_enqueue_style( 'fffilter_wp_admin_css' );
            wp_enqueue_script( 'fffilter_lottie', FFF_PLUGIN_URL . '/assets/js/lottie-player.js', array('jquery'), true );
            wp_enqueue_script( 'fffilter_wp_admin', FFF_PLUGIN_URL . '/assets/js/admin-scripts.js', array('jquery', 'wp-color-picker'), true );
            wp_localize_script( 'fffilter_wp_admin', 'fff_ajax_url', array('site_url' => get_site_url(), 'url' => admin_url( 'admin-ajax.php' )) );
            wp_localize_script( 'fffilter_wp_admin', 'fff_admin_page_url', array('url' => esc_url($this->admin_page_url)) );
            wp_localize_script( 'fffilter_wp_admin', 'fff_plugin_url', array('url' => FFF_PLUGIN_URL) );
        }
    }

    public function fff_add_menu_page(){
        add_menu_page( __( '3F Filter', 'fffilter' ), __( '3F Filters', 'fffilter' ), 'manage_options', 'fff-settings-page', array( $this, 'fff_menu_page_content' ), FFF_PLUGIN_URL . '/assets/img/icon.svg' );
        add_submenu_page( 'fff-settings-page', __( 'General settings', 'fffilter' ), __( 'General settings', 'fffilter' ), 'manage_options', 'fff-settings-general-page', array( $this, 'fff_menu_page_general_content' ) );
    }

    public function fff_menu_page_content(){
        $edit_option = (isset($_GET['edit'])) ? $_GET['edit'] : '';
        ?>
        <div class="wrap">
            <h1 class="fff-main-admin-title"><?php echo esc_html__('Fast Fancy Filters - 3F', 'fffilter'); ?></h1>
            <?php if(isset($_GET['new']) || isset($_GET['edit'])){ ?>
            <!-- New filter -->
            <form class="fff-new-filter-form" action="" method="post">
                <input type="hidden" name="fff_save_filter_settings" value="<?php echo esc_attr($edit_option); ?>" />
                <?php
                    include FFF_ABSPATH . 'includes/admin/templates/new-filter.php'; 
                ?>
            </form>
            <!-- New filter (END) -->
            <?php } else { ?>
            <a class="fff-button" href="<?php echo esc_url($this->admin_page_url) . '&new=1'; ?>"><?php echo esc_html__('Add new filter', 'fffilter'); ?></a>
            <!-- All filters -->
            <?php
                include FFF_ABSPATH . 'includes/admin/templates/all-filters.php'; 
            ?>
            <!-- All filters ( END ) -->
            <?php } ?>

            <div class="fff-saved-popup-cont fff-popup-save-filer">
                <div class="fff-saved-popup">
                    <lottie-player autoplay="true" loop="false" src="<?php echo FFF_PLUGIN_URL . '/assets/img/done.json' ?>" id="w8vav" speed="1" class="lottieanimation" background="transparent" style="width: 100%; height: 200px; overflow: hidden; margin: auto; cursor: default; background: transparent;"></lottie-player>
                    <p><?php echo esc_html__('Filter saved', 'fffilter'); ?></p>
                    <div class="fff-saved-popup-bttns">
                        <a class="fff-button fff-btn-blue" href="<?php echo esc_url($this->admin_page_url); ?>"><?php echo esc_html__('Back to all', 'fffilter'); ?></a>
                        <a class="fff-button fff-edit" href="#"><?php echo esc_html__('Edit filter', 'fffilter'); ?></a>
                    </div>
                </div>
            </div>

            <div class="fff-saved-popup-cont fff-popup-delete-filer">
                <div class="fff-saved-popup">
                    <lottie-player autoplay="true" loop="false" src="<?php echo FFF_PLUGIN_URL . '/assets/img/bin.json' ?>" id="w8vav" speed="1" class="lottieanimation" background="transparent" style="width: 100%; height: 200px; overflow: hidden; margin: auto; cursor: default; background: transparent;"></lottie-player>
                    <p><?php echo esc_html__('Are you sure?', 'fffilter'); ?></p>
                    <div class="fff-saved-popup-bttns">
                        <button class="fff-button fff-btn-blue fff-close-popup"><?php echo esc_html__('No', 'fffilter'); ?></button>
                        <button class="fff-button fff-delete-filter"><?php echo esc_html__('Yes', 'fffilter'); ?></button>
                    </div>
                </div>
            </div>

            <div class="fff-saved-popup-cont fff-popup-post-filer">
                <div class="fff-saved-popup">
                    <lottie-player autoplay="true" loop="false" src="<?php echo FFF_PLUGIN_URL . '/assets/img/question.json' ?>" id="w8vav" speed="1" class="lottieanimation" background="transparent" style="width: 100%; height: 200px; overflow: hidden; margin: auto; cursor: default; background: transparent;"></lottie-player>
                    <p><?php echo sprintf('%s<br />%s', __('Change post type?', 'fffilter'), __('All saved filter settings will be removed.')); ?></p>
                    <div class="fff-saved-popup-bttns">
                        <button class="fff-button fff-btn-blue fff-close-popup"><?php echo esc_html__('No', 'fffilter'); ?></button>
                        <button class="fff-button fff-change-post-filter"><?php echo esc_html__('Yes', 'fffilter'); ?></button>
                    </div>
                </div>
            </div>
        </div>
        <?php
    }

    public function fff_menu_page_general_content(){
        ?>
        <div class="wrap">
            <h1><?php echo esc_html__('3F - General settings', 'fffilter'); ?></h1>

            <div class="fff-wrap ff-filter-general-wrap">
                <form class="fff-general-filter-form" action="" method="post">
                    <input type="hidden" name="fff_save_filter_settings" value="general" />
                    <?php
                        $this->getFields($this->optionsFields('general_settings'));
                    ?>

                    <div class="fff-saved-popup-cont">
                        <div class="fff-saved-popup">
                            <lottie-player autoplay="true" loop="false" src="<?php echo FFF_PLUGIN_URL . '/assets/img/done.json' ?>" id="w8vav" speed="1" class="lottieanimation" background="transparent" style="width: 100%; height: 200px; overflow: hidden; margin: auto; cursor: default; background: transparent;"></lottie-player>
                            <p><?php echo esc_html__('Settings saved', 'fffilter'); ?></p>
                            <div class="fff-saved-popup-bttns center">
                                <a class="fff-button" href="<?php echo $_SERVER['REQUEST_URI']; ?>"><?php echo esc_html__('Okey', 'fffilter'); ?></a>
                            </div>
                        </div>
                    </div>
                    <button class="fff-button" type="submit"><?php echo esc_html__('Save settings', 'fffilter'); ?></button>
                </form>
            </div>
        </div>
        <?php
    }

    public function getFilter($filter_id = ''){
        $result = array();

        $args = array(
            'post_type' => 'ff_filter',
            'posts_per_page' => -1
        );

        if( $filter_id != '' ){
            if(get_post_status ( $edit_post_id )){
                $args['p'] = $filter_id;
            }
        }

        $query = new WP_Query( $args );
        $result = $query->get_posts();
        return $result;
    }

    public function getFields( $options, $post_id = '' ){
        foreach ( $options as $opt_k => $field ) {
            if ( ! isset( $field['type'] ) ) {
                return false;
            }
            $default_val = ( isset($field['default']) ) ? $field['default'] : '';
            if ( ! isset( $field['class'] ) ) {
				$field['class'] = '';
			}

            switch ( $field['type'] ) {
                // Text
                case 'text':
                case 'number':
                    $repeater_val = '';
                    if( isset($field['repeater_val']) ){
                        $repeater_val = $field['repeater_val'];
                    }
                    $option_value = (!isset($field['repeater_val_option'])) ? $this->getOption( $post_id, $field['id'], $default_val ) : $field['repeater_val_option'];

                    if($repeater_val != ''){
                        $v_id = esc_attr( $repeater_val );
                        $field['class'] .= ' repeater-field';
                    }else{
                        $v_id = esc_attr( $field['id'] );
                    }

                    if($option_value != ''){
                        $field['class'] .= ' active';
                    }
                    ?>
                    <div class="ff-filters-option">
                        <div class="ff-filters-option-input">
                            <input class="<?php echo esc_attr( $field['class'] ); ?>" id="<?php echo esc_attr($v_id); ?>" type="<?php echo esc_html($field['type']); ?>" name="<?php echo esc_attr($v_id); ?>" value="<?php echo esc_attr( $option_value ); ?>" />
                            <span class="bar"></span>
                            <label for="<?php echo esc_attr($v_id); ?>"><?php echo esc_html($field['label']); ?></label>
                        </div>
                    </div>
                    <?php
                    break;
                // Color
                case 'color':
                    $option_value = $this->getOption( $post_id, $field['id'], $default_val );
                    ?>
                    <div class="ff-filters-option fff-color-option">
                        <label class="ff-filters-option-label"><?php echo esc_html($field['label']); ?></label>
                        <input class="fff-color-field" id="<?php echo esc_attr($field['id']); ?>" type="text" name="<?php echo esc_attr($field['id']); ?>" value="<?php echo esc_attr( $option_value ); ?>" />
                    </div>
                    <?php
                    break;
                case 'codeedit':
                    $option_value = $this->getOption( $post_id, $field['id'], $default_val );
                    ?>
                    <div class="ff-filters-option fff-color-option">
                        <label class="ff-filters-option-label"><?php echo esc_html($field['label']); ?></label>
                        <textarea class="fff-code-field" name="<?php echo esc_attr($field['id']); ?>" id="<?php echo esc_attr($field['id']); ?>"><?php echo wp_unslash( $option_value ); ?></textarea>
                    </div>
                    <?php
                    break;
                // Image select
                case 'imageselect':
                    $option_value = $this->getOption( $post_id, $field['id'], $default_val );
                    ?>
                    <div class="ff-filters-option">
                        <div class="ff-filters-option-radio-wrap">
                            <label class="ff-filters-option-label"><?php echo esc_html($field['label']); ?></label>
                            <?php if(isset($field['options']) && is_array($field['options'])){ ?>
                            <div class="ff-filters-option-radio-cont">
                            <?php foreach($field['options'] as $opt_k => $option){ ?>
                                <div class="ff-filters-option-radio">
                                    <input type="radio" name="<?php echo esc_attr($field['id']); ?>" value="<?php echo esc_attr($opt_k); ?>" <?php checked( $option_value, esc_attr($opt_k) ); ?> />
                                    <img src="<?php echo FFF_PLUGIN_URL . '/assets/img/' . esc_html($option); ?>" />
                                </div>
                            <?php } ?>
                            </div>
                            <?php } ?>
                        </div>
                    </div>
                    <?php
                    break;
                // Switcher
                case 'switcher':
                    $option_value = $this->getOption( $post_id, $field['id'], $default_val );
                    ?>
                    <div class="ff-filters-option">
                        <div class="ff-filters-option-switcher">
                            <input id="<?php echo esc_attr($field['id']); ?>" type="checkbox" name="<?php echo esc_attr($field['id']); ?>" value="1" <?php checked( $option_value, 'yes' ); ?> />
                            <label for="check1">
                                <span><?php echo esc_html($field['label']); ?></span>
                                <div class="switch"></div> 
                            </label>
                        </div>
                    </div>
                    <?php
                    break;
                case 'select':
                    $repeater_val = '';
                    if( isset($field['repeater_val']) ){
                        $repeater_val = $field['repeater_val'];
                    }
                    $option_value = (!isset($field['repeater_val_option'])) ? $this->getOption( $post_id, $field['id'], $default_val ) : $field['repeater_val_option'];
                    $conditional_class = (isset($field['conditional_class'])) ? $field['conditional_class'] : '';
                    $conditional_data = (isset($field['conditional_data'])) ? $field['conditional_data'] : '';

                    if($repeater_val != ''){
                        $v_id = esc_attr( $repeater_val );
                        $field['class'] .= ' repeater-field';
                    }else{
                        $v_id = esc_attr( $field['id'] );
                    }

                    ?>
                    <div class="ff-filters-option <?php echo esc_attr($conditional_class); ?>" data-fffcond="<?php echo esc_attr($conditional_data); ?>">
                        <div class="ff-filters-option-select">
                            <?php if(isset($field['options']) && is_array($field['options'])){ ?>
                            <select data-default="<?php echo esc_attr( $default_val ); ?>" name="<?php echo $v_id; ?>" class="<?php echo esc_attr( $field['class'] ); ?>" required>
                                <?php foreach($field['options'] as $opt_k => $option){ ?>
                                <option value="<?php echo esc_attr($opt_k); ?>" <?php selected( $option_value, esc_attr($opt_k) ); ?>><?php echo esc_html($option); ?></option>
                                <?php } ?>
                            </select>
                            <?php } ?>
                            <span class="ff-filters-option-select-highlight"></span>
                            <span class="ff-filters-option-select-bar"></span>
                            <label><?php echo esc_html($field['label']); ?></label>
                        </div>
                    </div>
                    <?php
                    break;
                case 'checkboxset':
                    $option_value = $this->getOption( $post_id, $field['id'], $default_val );
                    ?>
                    <div class="ff-filters-option">
                        <label class="ff-filters-option-label"><?php echo esc_html($field['label']); ?></label>
                        <div class="ff-filters-checkbox-set">
                            <?php
                            if( !empty($field['options']) ){
                                foreach( $field['options'] as $check_k => $check ){
                                ?>
                                    <div class="ff-filters-checkbox-s">
                                        <input name="<?php echo esc_attr($field['id']); ?>[]" type="checkbox" value="<?php echo esc_attr($check_k); ?>" <?php if(in_array($check_k, $option_value)){echo 'checked';} ?> />
                                        <label>
                                            <span class="check"></span>
                                            <span><?php echo esc_html($check); ?></span>
                                        </label>
                                    </div>
                                <?php
                                }
                            }
                            ?>
                        </div>
                    </div>
                    <?php
                    break;
                case 'repeater':
                    $option_value = $this->getOption( $post_id, $field['id'], $default_val );
                    foreach( $field['options'] as $key => $val ){
                        $field['options'][$key]['repeater_val'] = $field['id'] . '[' . $val['id'] . '][]';
                        $field['options'][$key]['repeater_id'] = $field['id'];
                    }
                    ?>
                    <div class="ff-filters-option">
                        <label class="ff-filters-option-label"><?php echo esc_html($field['label']); ?></label>
                        <!-- items -->
                        <?php 
                            if( !empty($option_value) ){
                                foreach($option_value as $option_value_k => $option_value_v){
                                    foreach( $field['options'] as $key => $val ){
                                        $option_val = (isset($option_value_v[$val['id']])) ? $option_value_v[$val['id']] : '';
                                        $field['options'][$key]['repeater_val_option'] = $option_val;
                                        $conditional_data = $conditional = '';

                                        if( !empty($val['dependency']) ){
                                            foreach( $val['dependency'] as $cond_key => $cond_val ){
                                                $option_cond_val = (isset($option_value_v[$cond_key])) ? $option_value_v[$cond_key] : '';
                                                $cond_val_arr = explode(",", $cond_val);
                                                if( count($cond_val_arr) == 1 && strpos($cond_val_arr[0], '!=') !== false ){
                                                    $not_val = str_replace('!=', '', $cond_val_arr[0]);
                                                    if( $option_cond_val == $not_val ){
                                                        $conditional = 'fff-option-hide';
                                                    }
                                                } else {
                                                    if( !in_array($option_cond_val, $cond_val_arr) ){
                                                        $conditional = 'fff-option-hide';
                                                    }
                                                }
                                                
                                                $conditional_data .= $field['id'] . '[' . $cond_key . ']' . '[]|' . $cond_val . ';';
                                            }
                                        }

                                        if( $conditional_data != '' ){
                                            $conditional_data = rtrim($conditional_data, ";");
                                            $conditional_data = $conditional_data;
                                        }

                                        $field['options'][$key]['conditional_class'] = $conditional;
                                        $field['options'][$key]['conditional_data'] = $conditional_data;
                                    }
                                    ?>
                                    <div class="ff-filters-option-repeater-set <?php if($option_value_k == 0){echo 'f-set';} ?>">
                                        <?php if( $option_value_k > 0 ){ ?>
                                        <div class="ff-filters-option-del">
                                            <img src="<?php echo FFF_PLUGIN_URL . '/assets/img/delete.svg'; ?>" />
                                        </div>
                                        <?php } ?>
                                        <?php $this->getFields($field['options'], $post_id); ?>
                                    </div>
                                    <?php
                                }
                            } else {
                                foreach( $field['options'] as $key => $val ){
                                    $conditional_data = $conditional = '';
                                    if( !empty($val['dependency']) ){
                                        foreach( $val['dependency'] as $cond_key => $cond_val ){
                                            $conditional_data .= $field['id'] . '[' . $cond_key . ']' . '[]|' . $cond_val . ';';
                                            $conditional = 'fff-option-hide';
                                        }
            
                                        if( $conditional_data != '' ){
                                            $conditional_data = rtrim($conditional_data, ";");
                                            $conditional_data = $conditional_data;
                                        }
            
                                        $field['options'][$key]['conditional_class'] = $conditional;
                                        $field['options'][$key]['conditional_data'] = $conditional_data;
                                    }
                                }
                                ?>
                                <div class="ff-filters-option-repeater-set f-set">
                                    <?php $this->getFields($field['options'], $post_id); ?>
                                </div>
                                <?php
                            }
                        ?>
                        <!-- items -->

                        <button type="button" class="fff-add-rep">
                            <span><?php echo esc_html__('Add', 'fffilter'); ?></span>
                            <img src="<?php echo FFF_PLUGIN_URL . '/assets/img/add.svg'; ?>" />
                        </button>
                    </div>
                    <?php
                    break;

            }
        }
    }

    public function saveFields(){
        if ( isset($_POST['fff_save_filter_settings']) ) {
            $edit_post_id = sanitize_post( $_POST['fff_save_filter_settings'] );

            if( $edit_post_id == 'general' ){
                $options = $this->optionsFields('general_settings');
            } else {
                $options = $this->optionsFields();
            }

            if(empty($options)){
                return false;
            }

            $save_options = array();

            foreach ( $options as $option ) {
                if ( ! isset( $option['id'] ) || ! isset( $option['type'] ) ) {
                    continue;
                }

                $value = null;
                if ( strstr( $option['id'], '[' ) ) {
                    parse_str( $option['id'], $option_name_array );
                    $option_name = current( array_keys( $option_name_array ) );
                    $raw_value    = isset( $_POST[ $option_name ] ) ? sanitize_post( $_POST[ $option_name ] ) : null;
                } else {
                    $option_name  = $option['id'];
                    $setting_name = '';
                    $raw_value    = isset( $_POST[ $option['id'] ] ) ? sanitize_post( $_POST[ $option['id'] ] ) : null;
                }

                if( isset($raw_value['filter']) ){
                    unset($raw_value['filter']);
                }

                // Format the value based on option type.
                switch ( $option['type'] ) {
                    case 'switcher':
                        $value = '1' === $raw_value || 'yes' === $raw_value ? 'yes' : 'no';
                        break;
                    case 'repeater':
                        $rv = array();
                        if(!empty($raw_value)){
                            $count = count( reset($raw_value) ) - 1;
                            for($el = 0; $el <= $count; $el++){
                                $new_el = array();
                                foreach($raw_value as $raw_value_k => $raw_value_v){
                                    if(isset($raw_value_v[$el])){
                                        $new_el[$raw_value_k] = $raw_value_v[$el];
                                    }
                                }
                                array_push($rv, $new_el);
                            }
                        }
                        $value = array_filter( array_map( 'self::cleaWithHtml', (array) $rv ) );

                        break;
                    case 'checkboxset':
                        $value = array_filter( array_map( 'self::cleaWithHtml', (array) $raw_value ) );
                        break;
                    case 'codeedit':
                        $value = sanitize_textarea_field( $raw_value );
                        break;
                    default:
                        $value = sanitize_text_field( $raw_value );
                        break;
                }

                if ( is_null( $value ) ) {
                    continue;
                }

                $save_options[$option_name] = $value;
            }

            $errors = array();
            $post_id_return = 0; 

            if($edit_post_id != '' && $edit_post_id != 'general'){
                if ( get_post_status ( $edit_post_id ) ) {
                    update_post_meta($edit_post_id, 'fff_settings', $save_options);
                    $post_id_return = $edit_post_id; 
                } else {
                    $errors['not_exists'] = esc_html__('Post ID does not exists', 'fffilter');
                }
            } elseif($edit_post_id == 'general') {
                if( !empty($save_options) ){
                    foreach($save_options as $save_options_k => $save_options_v){
                        update_option( $save_options_k, $save_options_v );
                    }
                }
            } else {
                $new_post_id = wp_insert_post( array(
                    'post_title' => $save_options['filter_name'],
                    'post_status' => 'publish',
                    'post_type' => 'ff_filter'
                ) );
                update_post_meta($new_post_id, 'fff_settings', $save_options);
                $post_id_return = $new_post_id; 
            }

            if( empty($errors) ){
                echo json_encode( array('id' => $post_id_return) );
            } else {
                echo json_encode( $errors );
            }
        }
        
        exit;
    }

    public static function cleaWithHtml( $var ) {
        if ( is_array( $var ) ) {
            return array_map( 'self::cleaWithHtml', $var );
        } else {
            return is_scalar( $var ) ?  wp_kses_post( trim( $var ) ) : $var;
        }
    }

    public function removeFilter(){
        $res = 0;
        $filter_id = sanitize_post($_POST['filter']);
        if ( get_post_status ( $filter_id ) ) {
            wp_delete_post( $filter_id, true );
            $res = $filter_id;
        }

        echo $res;

        exit;
    }

    public function getTaxonomies(){
        $tax_arr = array();

        $post_type = sanitize_post($_POST['post_type']);

        $post_type_obj = get_post_type_object( $post_type );
        if( isset($post_type_obj->show_in_rest) && $post_type_obj->show_in_rest == 0 ){
            echo 'norest';
            exit;
        }

        if( $post_type == 'posts' ){
            $post_type = 'post';
            $tax_arr = array(
                'categories' => esc_html__('Category', 'fffilter'),
                'tags' => esc_html__('Tags', 'fffilter')
            );
        } else {
            $taxonomies = get_object_taxonomies( $post_type );
            if( !empty($taxonomies) ){
                foreach($taxonomies as $tax){
                    $s = get_taxonomy($tax);
                    $tax_q = $tax_l = '';
                    if( isset( $s->rest_base ) && $s->rest_base != '' ){
                        $tax_q = $s->rest_base;
                    } elseif( isset( $s->query_var ) && $s->query_var != '' ){
                        $tax_q = $s->query_var;
                    } else {
                        $tax_q = $s->name;
                    }

                    if( isset($s->label) ){
                        $tax_l = $s->label;
                    }

                    $tax_arr[$tax_q] = $tax_l;
                }
            }
        }

        $tax_arr = array_merge(array('search' => esc_html__('Searchbar', 'fffilter')), $tax_arr);
        foreach($tax_arr as $tax_arr_k => $tax_arr_v){
            ?>
            <option value="<?php echo esc_attr($tax_arr_k); ?>"><?php echo esc_html($tax_arr_v); ?></option>
            <?php
        }

        exit;
    }

    public function getOption( $post_id, $option_name, $default = '' ){
        $option_value = $default;
        if( $post_id != '' ){
            $options = maybe_unserialize( get_post_meta($post_id, 'fff_settings', true) );
            $option_value = ( isset($options[$option_name]) ) ? $options[$option_name] : null;
            if( null === $option_value ){
                $option_value = $default;
            }
        } else {
            $option_value = get_option( $option_name );
            if( $option_value == '' ){
                $option_value = $default;
            }
        }

        return $option_value;
    }

    public function optionsFields( $page = '', $post_id = '' ){
        $fields_result = array();
        $args_post_types = array(
            'public'   => true,
            '_builtin' => false,
        );

        $all_custom_posts = array();
        $all_custom_post_types = get_post_types($args_post_types, 'objects');

        if( !empty($all_custom_post_types) ){
            foreach( $all_custom_post_types as $all_custom_post_type ){
                $all_custom_posts[$all_custom_post_type->name] = $all_custom_post_type->label;
            }
        }

        $all_custom_posts = array_merge(array('posts' => esc_html__('Posts', 'fffilter')), $all_custom_posts);

        $post_type = '';
        if( $post_id != '' ){
            $post_type = $this->getOption( $post_id, 'post_types' );
        }

        $tax_arr = array(
            'categories' => esc_html__('Category', 'fffilter'),
            'tags' => esc_html__('Tags', 'fffilter')
        );

        if($post_type != 'posts'){
            $taxonomies = get_object_taxonomies( $post_type );
            if( !empty($taxonomies) ){
                $tax_arr = array();
                foreach($taxonomies as $tax){
                    $s = get_taxonomy($tax);
                    $tax_q = $tax_l = '';
                    if( isset( $s->rest_base ) && $s->rest_base != '' ){
                        $tax_q = $s->rest_base;
                    } elseif( isset( $s->query_var ) && $s->query_var != '' ){
                        $tax_q = $s->query_var;
                    } else {
                        $tax_q = $s->name;
                    }

                    if( isset($s->label) ){
                        $tax_l = $s->label;
                    }

                    $tax_arr[$tax_q] = $tax_l;
                }
            }
        }

        $tax_arr = array_merge(array('search' => esc_html__('Searchbar', 'fffilter')), $tax_arr);

        $new_form_general = array(
            array(
                'type' => 'text',
                'label' => esc_html__('Filter name', 'fffilter'),
                'id' => 'filter_name'
            ),
            array(
                'type' => 'select',
                'label' => esc_html__('Search in post types', 'fffilter'),
                'id' => 'post_types',
                'options' => $all_custom_posts,
                'default' => 'posts'
            ),
            array(
                'type' => 'imageselect',
                'label' => esc_html__('Filter position', 'fffilter'),
                'id' => 'filter_position',
                'options' => array(
                    'left' => 'grid-left.svg',
                    'right' => 'grid-right.svg',
                ),
                'default' => 'left'
            ),
            array(
                'type' => 'number',
                'label' => esc_html__('Results per page', 'fffilter'),
                'id' => 'posts_per_page',
                'default' => '10'
            ),
            array(
                'type' => 'select',
                'label' => esc_html__('Field relationships', 'fffilter'),
                'id' => 'field_relationships',
                'options' => array(
                    'AND' => esc_html__('And', 'fffilter'),
                    'OR' => esc_html__('Or', 'fffilter'),
                ),
                'default' => 'AND'
            ),
            array(
                'type' => 'select',
                'label' => esc_html__('Order posts by', 'fffilter'),
                'id' => 'order_by',
                'options' => array(
                    'date' => esc_html__('Date', 'fffilter'),
                    'title' => esc_html__('Title', 'fffilter'),
                    'modified' => esc_html__('Last Modified Date', 'fffilter'),
                ),
                'default' => 'date'
            ),
            array(
                'type' => 'select',
                'label' => esc_html__('Posts view', 'fffilter'),
                'id' => 'posts_view',
                'options' => array(
                    'masonry' => esc_html__('Masonry', 'fffilter'),
                    'classic' => esc_html__('Classic', 'fffilter'),
                ),
                'default' => 'masonry'
            ),
            array(
                'type' => 'switcher',
                'label' => esc_html__('Enable reset button', 'fffilter'),
                'id' => 'reset_button',
            ),
            array(
                'type' => 'switcher',
                'label' => esc_html__('Enable pagination', 'fffilter'),
                'id' => 'pagination',
            ),
            array(
                'type' => 'switcher',
                'label' => esc_html__('Sticky sidebar', 'fffilter'),
                'id' => 'sticky_sidebar',
            ),
        );

        $new_form_filters = array(
            array(
                'type' => 'repeater',
                'label' => esc_html__('Filters', 'fffilter'),
                'id' => 'filters',
                'options' => array(
                    array(
                        'type' => 'text',
                        'label' => esc_html__('Name', 'fffilter'),
                        'id' => 'filter_name',
                        'default' => 'Filter'
                    ),
                    array(
                        'type' => 'select',
                        'label' => esc_html__('Options', 'fffilter'),
                        'id' => 'filter_options',
                        'options' => $tax_arr,
                        'default' => 'search'
                    ),
                    array(
                        'type' => 'select',
                        'label' => esc_html__('Selection type', 'fffilter'),
                        'id' => 'filter_selection',
                        'options' => array(
                            'checkbox' => esc_html__('Checkbox', 'fffilter'),
                            'radio' => esc_html__('Radio', 'fffilter'),
                            'select' => esc_html__('Select', 'fffilter'),
                        ),
                        'default' => 'checkbox',
                        'dependency' => array( 'filter_options' => '!=search' )
                    )
                )
            )
        );

        $new_form_single_post = array(
            array(
                'type' => 'switcher',
                'label' => esc_html__('Feature image', 'fffilter'),
                'id' => 'single_feature_image',
                'default' => 'yes'
            ),
            array(
                'type' => 'switcher',
                'label' => esc_html__('Post title', 'fffilter'),
                'id' => 'single_post_title',
                'default' => 'yes'
            ),
            array(
                'type' => 'switcher',
                'label' => esc_html__('Post excerpt', 'fffilter'),
                'id' => 'single_post_excerpt',
                'default' => 'yes'
            ),
            array(
                'type' => 'switcher',
                'label' => esc_html__('Post meta', 'fffilter'),
                'id' => 'single_post_meta',
                'default' => 'yes'
            )
        );

        $general_settings = array(
            array(
                'type' => 'color',
                'label' => esc_html__('Main color', 'fffilter'),
                'id' => 'ff_filter_main_color',
                'default' => '#252a34'
            ),
            array(
                'type' => 'codeedit',
                'label' => esc_html__('Custom CSS', 'fffilter'),
                'id' => 'ff_filter_custom_css',
            )
        );

        if( $page == 'new_form_general' ){
            $fields_result = $new_form_general;
        }elseif( $page == 'new_form_filters' ){
            $fields_result = $new_form_filters;
        }elseif( $page == 'new_form_single_post' ){
            $fields_result = $new_form_single_post;
        }elseif( $page == 'general_settings' ){
            $fields_result = $general_settings;
        }else{
            $fields_result = array_merge($new_form_general, $new_form_filters, $new_form_single_post);
        }

        return $fields_result;
    }
}

return new FFFilter_Admin();