<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class FFFilter_Front {
    /**
	 * Class Constructor.
	 */
    public function __construct() {
        // Enqueue scripts
        add_action( 'wp_enqueue_scripts', array( $this, 'fff_enqueue_scripts' ) );
        
        // Add ff_filter post type
        add_action( 'init', array( $this, 'fff_register_post_type' ) );

        // Register the meta fields
        add_action( 'rest_api_init', array( $this, 'fff_register_meta_fields' ) );

        // Add shortcode
        add_shortcode( 'ff_filter', array($this, 'fff_shortcode') );

        add_action( 'init', array($this, 'fff_set_script_translations') );
    }

    public function fff_enqueue_scripts(){
        wp_register_style( 'fast-fancy-filter-main-css', FFF_PLUGIN_URL . '/assets/css/style.css', array(), FFF_VERSION );
        wp_enqueue_style( 'fast-fancy-filter-main-css' );

        // Main color
        $fff_main_color = get_option('ff_filter_main_color');
        $fff_custom_css = get_option('ff_filter_custom_css');
        $custom_css = '';
        if( $fff_custom_css != '' ){
            $custom_css .= $fff_custom_css;
        }
        if( $fff_main_color != '' ){
            $custom_css .= 'html:root{ --fff-main-color: '.$fff_main_color.'; }';
        }
        if($custom_css != ''){
            wp_add_inline_style( 'fast-fancy-filter-main-css', $custom_css );
        }

        wp_enqueue_script(
            'fast-fancy-filter-main-js',
            FFF_PLUGIN_URL . '/assets/app/main.js',
            array('wp-element', 'wp-components', 'wp-i18n'),
            '',
            true
        );

        wp_localize_script( 'fast-fancy-filter-main-js', 'main_site_url', array('url' => get_site_url()) );
        wp_localize_script( 'fast-fancy-filter-main-js', 'main_plugin_url', array('url' => FFF_PLUGIN_URL) );

        $structure = get_option( 'permalink_structure' );
        wp_localize_script( 'fast-fancy-filter-main-js', 'permalink_structure', array('val' => $structure) );
    }

    public function fff_shortcode( $atts = array() ){
        $main_id = (isset($atts['id'])) ? ' id="fffilter_'.$atts['id'].'"' : '';
        $id = uniqid( 'fffapp' );
        if( !empty($atts) ){
            wp_localize_script('fast-fancy-filter-main-js', "fffAppWpData_$id", $atts);
        }

        return '<div'.$main_id.' data-fff-app="'.$id.'"></div>';
    }

    public function fff_register_post_type(){
        register_post_type( 'ff_filter',
            array(
                'public' => false,
                'show_in_menu' => false,
                'labels' => array(
                    'name' => esc_html__('FFF filters', 'fffilter'),
                ),
                'show_in_rest' => true,
                'rewrite' => array('slug' => 'ff_filter', 'with_front' => false),
                'supports' => array( 'title', 'custom-fields' )
            )
        );
    }

    public function fff_register_meta_fields(){
        register_meta( 'post', 'fff_settings', array(
            'type' => 'string',
            'show_in_rest' => [
                'prepare_callback' => function( $value ) {
                    return wp_json_encode( maybe_unserialize($value) );
                }
            ],
        ));
    }

    public function getOption( $post_id, $option_name, $default = '' ){
        $option_value = $default;
        if( $post_id != '' ){
            $options = maybe_unserialize( get_post_meta($post_id, 'fff_settings', true) );
            $option_value = ( isset($options[$option_name]) ) ? $options[$option_name] : null;
            if( null === $option_value ){
                $option_value = $default;
            }
        }

        return $option_value;
    }

    public function fff_set_script_translations(){
        wp_set_script_translations( 'fast-fancy-filter-main-js', 'fffilter' );
    }
}

return new FFFilter_Front();